var exportModel = function(global)
{
    return exportBodies(global.bodies);
}
exportModel.filterName = "exportModel";
Library.addFilter("exportModel");


var dumpAll = function(global)
{
    return JSON.stringify(global, null, "\t");
}
dumpAll.filterName = "dumpAll";
Library.addFilter("dumpAll");


var exportBodies = function(bodies)
{
    var shapes = {
        generator_info: "Shape definitions generated with PhysicsEditor. Visit https://www.codeandweb.com/physicseditor"
    };
    for (var i = 0; i < bodies.length; i++)
    {
        var body = bodies[i];
        var fixtures = [];
        for (var j = 0; j < body.fixtures.length; j++)
        {
            var fixture = body.fixtures[j];
            var fixtureExport = exportFixtureParams(fixture);
            if (fixture.isCircle)
            {
                fixtureExport.circle = exportCircle(fixture);
            }
            else
            {
                fixtureExport.vertices = exportPolygons(fixture.polygons);
            }
            fixtures.push(fixtureExport);
        }

        var collisionFilter = {
            group: body.collisionFilterGroup,
            category: body.collisionFilterCategory,
            mask: body.collisionFilterMask
        };

        shapes[body.name] = {
            type: "fromPhysicsEditor",
            label: body.label ? body.label : body.name,
            isStatic: body.isStatic,
            density: body.density,
            restitution: body.restitution,
            friction: body.friction,
            frictionAir: body.frictionAir,
            frictionStatic: body.frictionStatic,
            collisionFilter: collisionFilter,
            fixtures: fixtures
        };
    }

    var str = JSON.stringify(shapes, null, "\t");
    return collapseJsonPoints(str);
}


var collapseJsonPoints = function(str) // removes whitespaces + line feeds around { x: .., y: .. }
{
    str = str.replace(/\n\s*{\n\s*"x":\s*([.-\d]+),\n\s*"y":\s*([.-\d]+)\n\s*}/g, " { \"x\":$1, \"y\":$2 }");
    str = str.replace(/(\n\s*\[[^\]]*)\n\s*\]/g, "$1 ]"); // move closing bracket up to the previous line
    return str;
}


var exportFixtureParams = function(fixture)
{
    var result = {
        label: fixture.label,
        isSensor: fixture.isSensor
    };
    return result;
}


var exportCircle = function(circle)
{
    var result = {
        x: circle.center.x,
        y: circle.center.y,
        radius: circle.radius
    }
    return result;
}


var exportPolygons = function(polygons)
{
    var result = [];
    for (var i = 0; i < polygons.length; i++)
    {
        var resultPoly = []
        for (var j = 0; j < polygons[i].length; j++)
        {
            resultPoly.push({
                x: polygons[i][j].x,
                y: polygons[i][j].y
            });
        }
        result.push(resultPoly);
    }
    return result;
}

